/*
 * Ovo je je C kod clienta koji odgovara
 * prostom ECHO serveru
 * 
 * Koristi sockete za komunikaciju,
 * errno sistem za ispisivanje
 * obavestenja i gresaka
 * 
 * Ima dva bafera, pri cemu jedan
 * prihvata korisnicki unos i sluzi
 * za slanje tih podataka preko socketa
 * dok drugi ima fiksnu velicinu i 
 * sluzi za prihvatanje podataka iz
 * sistemskog bafera
 * 
 * Za citanje sa stdin ulaza, koristi
 * se getline funkcija, jer scanf string
 * od vise reci deli prema razmaku, pa
 * se zbog toga i komunikacija poremeti
 * i izgubi se potpuna kontrola u pogledu
 * ko je sledeci koji salje a ko prima podatke.
 * 
 * Ne moramo da vodimo racuna o '\0' 
 * karakteru na kraju stringa posto
 * se na njegovom kraju uvek nadje '\n'
 * zbog nacina na koji se snima unos
 * korisnicke poruke
 * 
 * Kodirao: Ikac, e-mail: ikac.ikax@gmail.com
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>

int sockfd;

void sigpipe()
{
  //kada klijent pokusa da pise u socket koji je
  //sa druge strane zatvoren, ispisace informaciju
  //i zatvoriti svoju stranu
	printf("Zatvaram konekciju");
	close(sockfd);
}

int main(int argc, char *argv[])
{
	int portno, n;
	struct sockaddr_in serv_addr;
	struct hostent *server;

	signal(SIGPIPE, sigpipe);

	if (argc < 3) {
		fprintf(stderr, "uputstvo: %s hostname port\n",argv[0]);
		exit(1);
	}

	portno = atoi(argv[2]);

	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	perror("Otvaranje socket-a");
	if (sockfd < 0)
		exit(1);

	server = gethostbyname(argv[1]);
	perror("Trazenje servera");

	if (!server)
		exit(1);

	memset(&serv_addr, '0', sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;	//koriste se Internet adrese
	memcpy(&serv_addr.sin_addr.s_addr, server->h_addr, server->h_length);	//uzima se adresa servera
	serv_addr.sin_port = htons(portno);	//uzima se port naveden kao argument pri pokretanju

	// ZAVRSETAK FIKSNOG DELA

	n = connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr));
	perror("Konektovanje na server");
	if (n < 0)
		exit(1);

	char *buffer_out;	//skladiste za podatke koji se salju
	char buffer_in[256];	//skladiste za podatke koje cemo primiti
	int nbytes;		//broj bajtova koje smo primili u skladiste za prijem

	while (1) {
		printf("Unesi poruku: ");
		getline(&buffer_out, &nbytes, stdin);	//prihvata korisnicki ulaz i vraca koliko je bajtova primljeno u nbytes promenljivu

		if (nbytes > 256)
			printf
			    ("Poruka je ogranicena na 256 karaktera. Ostatak ce biti zanemaren\n");
		n = write(sockfd, buffer_out, 256);	//slanje podataka iz skladista u koje smo upisali korisnicki unos
		perror("Slanje poruke");

		if (n < 0)
			break;

		memset(buffer_in, ' ', 256);	//nulovanje celog skladista za prijem podataka
		n = read(sockfd, buffer_in, 256);	//prihvatanje podataka u skladiste za prijem
		perror("Prihvatanje poruke");

		if (n < 0)
			break;

		printf("Server: %s\n", buffer_in);

	}
	close(sockfd);

	return 0;
}
